 /**********************************************************************
 *
 *
 *		            Rover Toolkit
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: roverhttp.h
 *
 * Description: Rover server-side HTTP proxy routines.
 *
 *
 * Global Functions: None
 * Global Variables: None
 *
 * 
 *
 ***********************************************************************
 *   Copyright 1995-1998
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#ifndef ROVER_HTTP_H
#define ROVER_HTTP_H

typedef enum _HTTPState {
  HTTP_BEGIN = 0,
  HTTP_NEED_CONNECTION,
  HTTP_NEED_REQUEST,
  HTTP_SENT_REQUEST,
  HTTP_NEED_BODY,
  HTTP_GOT_BODY,
  HTTP_CLEANUP,
  HTTP_ABORT,
} HTTPState;

/* All the data sent from the access manager in the MIME headers. It has
   the same names here as in the access manager (HTRoverHTTP.c)  */
struct _HTTPInfo {
  char *pointer;	/* Pointer to current input point */
  char *source;		/* Source URL of requested document (Referer) */
  char *sourcetitle;	/* Source title of requested document (if known) */
  char *path;		/* Client storage path */
  char *user;		/* Client user */
  char *pagetitle;	/* Parsed title of requested document */
  char *method;		/* Client request method */
  char *url;		/* URL of requested document */
  char *rhost;		/* Client hostname or IP addr */
  char *MIMEHdr;	/* Client MIME header */
  int logID;		/* Log ID of client request */ 
  int rport;		/* Client httpd port */
  int prefetch;		/* Depth of prefetching */
  int doinline;		/* Inline image prefetching flag */
  int intitle;		/* Flag indicating HTML parser is in document title */
  int visible;		/* Flag indicating whether document is visible */
  int docompress;	/* Compression level */
  time_t reqTime;	/* Time of client request */
  time_t respTime;	/* Time of server response */
  long preftime;	/* Prefetch delay */
  int first;		/* First request */
  FILE *osockout;	/* Output file handle */
  int osock;		/* Output socket */
  char *reply;		/* Reply for SMTP */
  int SMTP;		/* SMTP request */
  char *wwwcmd;		/* WWW linemode browser command */
  char *mailcmd;	/* Sendmail command */
  HTTPState state;	/* State of the proxy */
};
typedef struct _HTTPInfo HTTPInfo;



#endif /* ROVER_HTTP_H */
