 /**********************************************************************
 *
 *
 *		            Rover Toolkit
 *			  Anthony D. Joseph
 *		   Laboratory for Computer Science
 *		Massachusetts Institute of Technology
 *
 * Module: rovertcp.c
 *
 * Description: Rover Server TCP module.
 *
 *
 * Global Functions: None
 * Global Variables: None
 *
 * 
 *
 ***********************************************************************
 *   Copyright 1995-1998
 *   Anthony D. Joseph
 *   Massachusetts Institute of Technology
 *
 *   Permission to use, copy, modify, and distribute this program
 *   for any purpose and without fee is hereby granted, provided
 *   that this copyright and permission notice appear on all copies
 *   and supporting documentation, the name of M.I.T. not be used
 *   in advertising or publicity pertaining to distribution of the
 *   program without specific prior permission, and notice be given
 *   in supporting documentation that copying and distribution is
 *   by permission of M.I.T.  M.I.T. makes no representations about
 *   the suitability of this software for any purpose.  It is pro-
 *   vided "as is" without express or implied warranty.		
 ***********************************************************************/

#include <sys/types.h>
#include <sys/socket.h>
#include <netinet/in.h>
#include <arpa/inet.h>
#ifndef linux
#include <sys/filio.h>
#include <netinet/tcp.h>         /* for TCP_MAXSEG value */
#else
#include <stdarg.h>
#include <unistd.h>
#endif /* !linux */
#include <netdb.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <ctype.h>
#include "rover.h"
#include "rovertcp.h"
#include "zlib.h"

PUBLIC ServerState *state;

/* Send header information to a client. */
PUBLIC void sendHeader(FILE *sock, char *url, int compress, int image,
		       int logID, char *source, char *path,
		       unsigned long time1, unsigned long time2,
		       unsigned long time3, int visible, int prefetch)
{
  fprintf(sock,"url: %s\n",url);
  fprintf(sock,"compress: %d\n",compress);
  fprintf(sock,"image: %s\n", image ? "yes" : "no");
  fprintf(sock,"logid: %d\n",logID);	/* optional? */
  if (source) fprintf(sock,"source: %s\n",source);
  if (path) fprintf(sock,"path: %s\n",path);
  fprintf(sock,"time1: %lu\n",(unsigned long) time1);
  fprintf(sock,"time2: %lu\n",(unsigned long) time2);
  fprintf(sock,"time3: %lu\n",(unsigned long) time3);
  fprintf(sock,"visible: %s\n",visible ? "yes" : "no");
  fprintf(sock,"prefetch: %d\n",prefetch);
  fprintf(sock,"\n");
  fflush(sock);
}


/* Send mail (SMTP) header information to a client. */
PUBLIC void sendMailHeader(FILE *file, char *url, char *origUrl,
			   int docompress, int image, char *rhost, int rport,
			   int logID, char *source, char *path, char *user,
			   unsigned long time1, unsigned long time2,
			   int prefetch)
{
  fprintf(file,"MIME-version: 1.0\n"); /* make it MIME-compliant */
  fprintf(file,"X-URL: %s\n",url);
  fprintf(file,"X-Compress: %d\n",docompress);
  fprintf(file,"X-image: %s\n",image ? "yes" : "no");
  if (rhost) fprintf(file,"X-hostname: %s\n",rhost);
  if (rport) fprintf(file,"X-port: %d\n",rport);
  if (!strcmp(url,origUrl)) fprintf(file,"X-logID: %d\n", logID);
  if (source) fprintf(file,"X-source: %s\n",source);
  if (path) fprintf(file,"X-path: %s\n",path);
  if (user) fprintf(file,"X-user: %s\n",user);
  fprintf(file,"X-time1: %lu\n",(unsigned long) time1);
  fprintf(file,"X-time2: %lu\n",(unsigned long) time2);
  fprintf(file,"X-prefetch: %d\n", prefetch);
  fprintf(file,"Content-type: X-web-response\n");
  fprintf(file,"Content-encoding: base64\n");
  fprintf(file,"\n");
  fflush(file);
}

/*
 * Return the text string associated with a status code
 */
PUBLIC char *StatusMsg(Codes code)
{
  switch(code) {
  case OK:
    return("OK");
    break;
  case CREATED:
    return("Created");
    break;
  case ACCEPTED:
    return("Accepted");
    break;
  case PROVIS_INFO:
    return("Provisional Information");
    break;
  case NO_RESPONSE:
    return("No Response");
    break;
  case DELETED:
    return("Deleted");
    break;
  case MODIFIED:
    return("Modified");
    break;
  case MOVED_PERM:
    return("Moved Permanently");
    break;
  case MOVED_TEMP:
    return("Moved Temporarily");
    break;
  case METHOD:
    return("Method");
    break;
  case NOT_MODIFIED:
    return("Not Modified");
    break;
  case BAD_REQUEST:
    return("Bad Request");
    break;
  case UNAUTHORIZED:
    return("Unauthorized");
    break;
  case PAYMENT_REQUIRED:
    return("Payment Required");
    break;
  case FORBIDDEN:
    return("Forbidden");
    break;
  case NOT_FOUND:
    return("Not Found");
    break;
  case METHOD_NOT_ALLOW:
    return("Method Not Allowed");
    break;
  case NONE_ACCEPTABLE:
    return("None Acceptable");
    break;
  case PROXY_AUTH_REQ:
    return("Proxy Authentication Required");
    break;
  case REQUEST_TIMEOUT:
    return("Request Timeout");
    break;
  case INTERNAL_ERROR:
    return("Internal Error");
    break;
  case NOT_IMPLEMENTED:
    return("Not Implemented");
    break;
  case BAD_GATEWAY:
    return("Bad Gateway");
    break;
  case SERVICE_UNAVAIL:
    return("Service Unavailable");
    break;
  case GATEWAY_TIMEOUT:
    return("Gateway Timeout");
    break;
  default:
    return("Unknown status code");
  }
}
